/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F3910_LVD
 * IDE tooling  : AndeSight C/C++ IDE, version: 3.2.1
 * Device Ver.  :
 * Library Ver. : 0.5
 * MCU Device   :
 * Description  :
 * Created Date : 2022/03/22
 * Created by   :
 *
 * Program Description:
 *
 *   HY16F3910                LCD Board(HY10000-AM01)                                                                       
 *----------------          ----------------
 *                |        |                                             
 *  PT13.4~PT13.5 | -----> | SEG0~SEG1                                                                                                             
 *    PT6.0~PT6.7 | -----> | SEG2~SEG9                                                                                                             
 *    PT7.0~PT7.7 | -----> | SEG10~SEG17                                                                                                                                                                                                           
 *  PT13.0~PT13.3 | -----> | COM0~COM3                                                                                                             
 *                |        |                                             
 *----------------         ----------------  
 * 
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY391apb.h"
#include "DrvLCD.h"
#include "DrvClock.h"
#include "Display.h"
#include "main.h"
#include "System.h"
#include "DrvPMU.h"
/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#define VOLTAGE_High  1
#define VOLTAGE_LOW   0

//OSC
//#define HSXT  //External 16MHz
#define HSRC  //Internal HAO
//#define HAO_4MHZ
#define HAO_32MHZ
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);

/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{
  unsigned char LVDO_JUDGEMENT;
  DrvPMU_VDD15Trim();  // Make sure VDD15 Voltage,before change OSC setting
#if defined(HSRC)
	  DrvCLOCK_EnableHighOSC(E_INTERNAL,100);     // Select HSRC
	#if defined(HAO_4MHZ)
	  DrvCLOCK_SelectIHOSC_CalHAO(E_HAO_4M);                 //Select internal 4.147MHz, and calibration 4.147MHz
	  DrvCLOCK_SelectMCUClock(E_HSCK,MCUCKDIV1);                //ENMCD=01b,MCU Clock/1, CPU CLOCK IS 'HS_CK/1'
	#elif defined(HAO_32MHZ)
	  DrvCLOCK_SelectIHOSC_CalHAO(E_HAO_32M);                 //Select internal 31.795MHz, and calibration 31.795MHz
	  DrvCLOCK_SelectMCUClock(E_HSCK,MCUCKDIV2);                //ENMCD=00b,MCU Clock/2, CPU CLOCK IS 'HS_CK/2'
	#endif
#endif
#if defined(HSXT)
  DrvCLOCK_SelectOHS_HS(1);   //0:HSXT<4MHZ 1:HSXT>4MHZ
  DrvCLOCK_EnableHighOSC(E_EXTERNAL,50);
#endif

  DisplayInit();
  ClearLCDframe();
  Delay(10000);
  DisplayHYcon();

  //Set bandgap voltage
  DrvPMU_VDDA_LDO_Ctrl(E_LDO);
  DrvPMU_VDDA_Voltage(E_VDDA2_4);
  DrvPMU_BandgapEnable();

  //Set LVD
  DrvPMU_SetLVDS(LVD_24V);
  DrvPMU_SetLVD12(LVD_V12_BGR);
  DrvPMU_SetLVDVS(LVD_VDD5V);
  DrvPMU_EnableENLVD();

  while(1)
  {
    LVDO_JUDGEMENT=DrvPMU_GetLVDO();
    if(LVDO_JUDGEMENT==VOLTAGE_High)
    {
       LCD_DATA_DISPLAY(VOLTAGE_High);  //VDD5V>LVDS
    }
    if(LVDO_JUDGEMENT==VOLTAGE_LOW)
    {
       LCD_DATA_DISPLAY(VOLTAGE_LOW);   //VDD5V<LVDS
    }
  }

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : LVD & BOR2 interrupt Service Routine (HW3).                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW7_ISR()                                                   */
/* Description  : UART2 interrupt Service Routine (HW7).                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW7_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW8_ISR()                                                   */
/* Description  : TMB2 interrupt Service Routine (HW8).                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW8_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW9_ISR()                                                   */
/* Description  : PT3 interrupt Service Routine (HW9).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW9_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: general_exception_handler()                                 */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void general_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
  while(1);
}
/*----------------------------------------------------------------------------*/
/* Function Name: debug_exception_handler()                                   */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void debug_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
  while(1);
}
/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}

/*----------------------------------------------------------------------------*/
/* End Of File                                                               */
/*----------------------------------------------------------------------------*/
